from sklearn.linear_model import LogisticRegression as LogR, LinearRegression as LinR
from pyspark.ml.classification import LogisticRegression as SparkLogR
from pyspark.ml.regression import LinearRegression as SparkLinR


class PredictorBase:
	def __init__(
			self, model, x_columns=None, y_column=None, normalize=False
	):
		"""
		:type model: LogR or LinR or SparkLogR or SparkLinR
		:type x_columns: str or list[str] or tuple or set or NoneType
		:type y_column: str or list[str] or tuple or set or NoneType
		"""
		self._untrained_model = None
		self._trained_model = None
		self._x_columns = None
		self._y_column = None
		self._normalize = normalize

		if x_columns is not None:
			self.x_columns = x_columns

		if y_column is not None:
			self.y_column = y_column

		self.untrained_model = model

	@property
	def x_columns(self):
		"""
		:rtype: list[str]
		"""
		return self._x_columns

	@x_columns.setter
	def x_columns(self, x_columns):
		"""
		:type x_columns: str or list[str] or tuple or set
		"""
		if self._x_columns is not None:
			raise RuntimeError('x_columns has already been set!')

		if isinstance(x_columns, str):
			x_columns = [x_columns]
		elif isinstance(x_columns, (tuple, set)):
			x_columns = list(x_columns)
		elif not isinstance(x_columns, list):
			raise TypeError(f'x_columns of type {type(x_columns)} is not supported!')

		if not all([isinstance(col, str) for col in x_columns]):
			raise TypeError(f'x_columns should be strings!')

		self._x_columns = x_columns

	@property
	def y_column(self):
		"""
		:rtype: str
		"""
		return self._y_column

	@y_column.setter
	def y_column(self, y_column):
		"""
		:type y_column: str or list[str] or tuple or set
		"""
		if self._y_column is not None:
			raise RuntimeError('y_column has already been set!')

		if isinstance(y_column, (set, tuple, list)):
			if len(y_column) > 1:
				raise ValueError('Too many values for y_column!')
			y_column = y_column[0]

		if not isinstance(y_column, str):
			raise TypeError(f'y_column of type {type(y_column)} is not supported!')

		self._y_column = y_column

	@property
	def untrained_model(self):
		"""
		:rtype:
		"""
		return self._untrained_model

	@untrained_model.setter
	def untrained_model(self, untrained_model):
		"""
		:type untrained_model:
		"""
		if self._untrained_model is not None:
			raise RuntimeError('untrained model has already been set!')

	@property
	def trained_model(self):
		"""
		:rtype:
		"""
		return self._trained_model



import tldextract


def get_domain_and_tld(url):
	"""
	return the domain and tld: forums.news.cnn.com/index.html --> cnn.com, http://bbc.co.uk --> bbc.co.uk
	:type url: str
	:rtype: str
	"""
	if url is None:
		return None

	ext = tldextract.extract(url)
	return f'{ext.domain}.{ext.suffix}'



from pyspark.sql import DataFrame as SparkDF
from pyspark.sql import functions as f
from pyspark.sql.types import StringType, BooleanType
from pandas import DataFrame as PandasDF
import validators
from .union import union
from ...text import get_domain_and_tld


@f.udf(StringType())
def get_domain_and_tld_column(url):
	if url is not None:
		return get_domain_and_tld(url)
	else:
		return None


def find_values(data, value_types):
	"""
	finds domains, emails, or ips
	returns a dataframe with three columns: column_name, value_type, value
	if df has columns: parent_domain, email_address, ip_address, child_domain
	with values that match those
	if value_type=['domain']
	the returning dataframe will have column_name with values like parent_domain and child_domain
	value_type as domain
	with domains from those columns in the value column

	:type data: PandasDF or SparkDF
	:type value_types: list[str] or str
	:param value_types: url, domain, email, ip_address
	:rtype: PandasDF or SparkDF
	"""
	if isinstance(value_types, str):
		value_types = [value_types]
	if value_types is None:
		value_types = ['domain', 'email', 'ip_address', 'url']

	dataframes = []

	if isinstance(data, SparkDF):

		for value_type in value_types:
			if value_type == 'domain':
				@f.udf(StringType())
				def get_value_column(x):
					try:
						return get_domain_and_tld(x)
					except:
						return None
			else:
				get_value_column = None

			is_valid = getattr(validators, value_type)

			@f.udf(BooleanType())
			def get_is_valid_column(x):
				try:
					if is_valid(x):
						return True
					else:
						return False
				except:
					return False

			for col in data.columns:

				if get_value_column is None:
					df = data.select(
						f.lit(value_type).alias('value_type'), f.lit(col).alias('column'),
						f.col(col).alias('value')
					)
				else:
					df = data.select(
						f.lit(value_type).alias('value_type'), f.lit(col).alias('column'),
						get_value_column(col).alias('value')
					)

				df = df.withColumn('is_valid', get_is_valid_column('value'))
				dataframes.append(df)

		return union(*dataframes, n_jobs=4).filter(f.col('is_valid')).drop('is_valid')

	elif isinstance(data, PandasDF):

		for value_type in value_types:
			if value_type == 'domain':
				def get_value_column(x):
					try:
						return get_domain_and_tld(x)
					except:
						return None
			else:
				get_value_column = None

			_is_valid = getattr(validators, value_type)
			def is_valid(x):
				try:
					result = _is_valid(x)
					return result == True
				except:
					return False

			for col in data.columns:
				df = data[[col]].copy()
				df['value_type'] = value_type
				df['column'] = col
				if get_value_column is None:
					df['value'] = df[col]
				else:
					df['value'] = df[col].apply(get_value_column)

				df['is_valid'] = df['value'].apply(is_valid)
				dataframes.append(df.drop(columns=[col]))

		result = union(*dataframes, n_jobs=4)
		return result[result['is_valid']].reset_index(drop=True)

	else:
		raise TypeError(f'data of type {type(data)} is not supported!')
